package cn.com.qmth.stmms.biz.exam.service.impl;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.MarkGroupDao;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.MarkGroupPK;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

@Service("markGroupService")
public class MarkGroupServiceImpl extends BaseQueryService<MarkGroup> implements MarkGroupService {

    @Autowired
    private MarkGroupDao groupDao;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private TaskService taskService;

    @Transactional
    @Override
    public MarkGroup save(MarkGroup group) {
        if (group.getQuestionList() != null && !group.getQuestionList().isEmpty()) {
            questionService.deleteByExamAndSubjectAndObjectiveAndMainNumber(group.getExamId(), group.getSubjectCode(),
                    false, group.getNumber());
            for (ExamQuestion question : group.getQuestionList()) {
                questionService.save(question);
            }
        }
        return groupDao.save(group);
    }

    @Transactional
    @Override
    public void reset(MarkGroup group) {
        groupDao.resetCount(group.getExamId(), group.getSubjectCode(), group.getNumber());

        libraryService.resetByGroup(group);
        taskService.clearCurrent(group.getExamId(), group.getSubjectCode(), group.getNumber());
    }

    @Transactional
    @Override
    public void resetBuildTime(int examId, String subjectCode) {
        groupDao.updateBuildTime(examId, subjectCode, null);
    }

    @Transactional
    @Override
    public void delete(MarkGroup group) {
        groupDao.delete(group);

        taskService.clearCurrent(group.getExamId(), group.getSubjectCode(), group.getNumber());
        questionService.deleteByExamAndSubjectAndObjectiveAndMainNumber(group.getExamId(), group.getSubjectCode(),
                false, group.getNumber());
        libraryService.deleteByGroup(group);
        markerService.deleteByGroup(group);
        subjectService.updateScore(group.getExamId(), group.getSubjectCode());
    }

    @Transactional
    @Override
    public void updateLibraryCount(int examId, String subjectCode, int number) {
        groupDao.updateLibraryCount(examId, subjectCode, number);
        groupDao.updateMarkedCount(examId, subjectCode, number, LibraryStatus.MARKED);
        groupDao.updateLeftCount(examId, subjectCode, number);
    }

    @Transactional
    @Override
    public void updateLibraryCount(int examId, String subjectCode) {
        groupDao.updateLibraryCount(examId, subjectCode);
        groupDao.updateMarkedCount(examId, subjectCode, LibraryStatus.MARKED);
        groupDao.updateLeftCount(examId, subjectCode);
    }

    @Transactional
    @Override
    public void updatePicList(int examId, String subjectCode, int number, List<Integer> picList) {
        groupDao.updatePicList(examId, subjectCode, number,
                picList != null && picList.size() > 0 ? StringUtils.join(picList, MarkGroup.PIC_SPLIT) : "");
    }

    @Transactional
    @Override
    public void updateTitle(int examId, String subjectCode, int number, String title) {
        groupDao.updateTitle(examId, subjectCode, number, title);
        questionService.updateMainTitle(examId, subjectCode, false, number, title);
    }

    @Transactional
    @Override
    public void updateTotalScore(int examId, String subjectCode, int number, Double score) {
        groupDao.updateTotalScore(examId, subjectCode, number, score);
    }

    @Transactional
    @Override
    public void updateBuildTime(int examId, String subjectCode, int number, Date time) {
        groupDao.updateBuildTime(examId, subjectCode, number, time);
    }

    @Transactional
    @Override
    public void deleteByExam(int examId) {
        groupDao.deleteByExamId(examId);
    }

    @Transactional
    @Override
    public void deleteByExamAndSubject(int examId, String subjectCode) {
        groupDao.deleteByExamIdAndSubjectCode(examId, subjectCode);
    }

    @Override
    public MarkGroup findOne(int examId, String subjectCode, int number) {
        MarkGroupPK pk = new MarkGroupPK();
        pk.setExamId(examId);
        pk.setSubjectCode(subjectCode);
        pk.setNumber(number);
        return groupDao.findOne(pk);
    }

    @Override
    public long countByExamAndSubject(int examId, String subjectCode) {
        return groupDao.countByExamIdAndSubjectCode(examId, subjectCode);
    }

    @Override
    public List<MarkGroup> findByExam(int examId) {
        return groupDao.findByExamId(examId);
    }

    @Override
    public List<MarkGroup> findByExamAndSubject(int examId, String code) {
        return groupDao.findByExamIdAndSubjectCode(examId, code);
    }

    @Override
    public long countByExam(int examId) {
        return groupDao.countByExamId(examId);
    }

}
