package cn.com.qmth.stmms.biz.mark.dao;

import java.util.List;
import java.util.Set;

import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;

import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

public interface MarkLibraryDao
        extends PagingAndSortingRepository<MarkLibrary, Integer>, JpaSpecificationExecutor<MarkLibrary> {

    List<MarkLibrary> findByExamId(int examId, Pageable page);

    List<MarkLibrary> findByExamIdAndSubjectCode(int examId, String subjectCode, Pageable page);

    List<MarkLibrary> findByExamIdAndStatus(int examId, LibraryStatus status, Pageable page);

    List<MarkLibrary> findByExamIdAndSubjectCodeAndGroupNumberAndStatus(int examId, String subjectCode, int groupNumber,
            LibraryStatus status, Pageable page);

    List<MarkLibrary> findByExamIdAndSubjectCodeAndGroupNumberAndStatusIn(int examId, String subjectCode,
            int groupNumber, Set<LibraryStatus> statusSet, Pageable page);

    List<MarkLibrary> findByExamIdAndMarkerId(int examId, int markerId, Pageable page);

    List<MarkLibrary> findByExamIdAndExamNumber(int examId, String examNumber);

    @Query("select l from MarkLibrary l where l.studentId=?1 order by l.groupNumber ")
    List<MarkLibrary> findByStudentId(int studentId);

    List<MarkLibrary> findByStudentIdAndGroupNumber(int studentId, int groupNumber);

    @Query("select distinct l.campusId from MarkLibrary l where l.examId=?1 and l.tags is not null")
    List<Integer> findTagCampusId(int examId);

    @Query("select distinct l.subjectCode from MarkLibrary l where l.examId=?1 and l.tags is not null")
    List<String> findTagSubjectCode(int examId);

    @Modifying
    @Query("update MarkLibrary m set m.status=?2, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null where m.examId=?1")
    void resetByExamId(int examId, LibraryStatus status);

    @Modifying
    @Query("update MarkLibrary m set m.status=?4, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null where m.examId=?1 and m.subjectCode=?2 and m.groupNumber=?3")
    void resetByExamIdAndSubjectCodeAndNumber(int examId, String subjectCode, int number, LibraryStatus status);

    @Modifying
    @Query("update MarkLibrary m set m.status=?2, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null where m.markerId=?1")
    void resetByMarkerId(int markerId, LibraryStatus status);

    @Modifying
    @Query("update MarkLibrary m set m.status=?2, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null where m.id=?1")
    void resetById(int id, LibraryStatus status);

    @Query("select count(*) from MarkLibrary f where f.examId=?1")
    long countByExamId(int examId);

    @Query("select count(*) from MarkLibrary f where f.examId=?1 and f.subjectCode=?2")
    long countByExamIdAndSubjectCode(int examId, String subjectCode);

    @Query("select count(*) from MarkLibrary f where f.examId=?1 and f.subjectCode=?2 and f.status=?3")
    long countByExamIdAndSubjectCodeAndStatus(int examId, String subjectCode, LibraryStatus status);

    @Query("select f.markerId, count(*) as markerCount from MarkLibrary f where f.examId=?1 and f.status=?2 group by f.markerId")
    List<Object[]> countByMarkerAndStatus(int examId, LibraryStatus status);

    @Query("select f.markerId, count(*) as markerCount from MarkLibrary f where f.examId=?1 and f.subjectCode=?2 and f.status=?3 group by f.markerId")
    List<Object[]> countByMarkerAndStatus(int examId, String subjectCode, LibraryStatus status);

    @Modifying
    @Query("delete MarkLibrary m where m.examId=?1")
    void deleteByExamId(int examId);

    @Modifying
    @Query("delete MarkLibrary m where m.examId=?1 and m.subjectCode=?2 and m.groupNumber=?3")
    void deleteByExamIdAndSubjectCodeAndGroupNumber(int examId, String subjectCode, int groupNumber);

    @Query("select l from MarkLibrary l where l.studentId=?1 and l.status=?2 order by l.groupNumber ")
    List<MarkLibrary> findByStudentIdAndStatus(int studentId, LibraryStatus status);

}
