package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.mark.dao.MarkLibraryDao;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

@Service
public class MarkLibraryServiceImpl extends BaseQueryService<MarkLibrary> implements MarkLibraryService {

    @Autowired
    private MarkLibraryDao libraryDao;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private ExamQuestionService questionService;
    
    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Transactional
    @Override
    public void delete(MarkLibrary library) {
    	markSpecialTagService.deleteByLibraryId(library.getId());
        libraryDao.delete(library);
        if (library != null) {
            List<ExamQuestion> examQuestions = questionService.findByExamAndSubjectAndObjectiveAndMainNumber(
                    library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
            for (ExamQuestion examQuestion : examQuestions) {
                trackService.deleteByStudentIdAndQuestionNumber(library.getStudentId(),
                        examQuestion.getQuestionNumber());
            }
        }
    }

    @Transactional
    @Override
    public void deleteByStudent(int studentId) {
        List<MarkLibrary> library = findByStudentId(studentId);
        if (library != null) {
            for (MarkLibrary markLibrary : library) {
            	markSpecialTagService.deleteByLibraryId(markLibrary.getId());
                this.delete(markLibrary);
            }
        }
        trackService.deleteByStudentId(studentId);
    }

    @Transactional
    @Override
    public void deleteByGroup(MarkGroup group) {
    	markSpecialTagService.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
        libraryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        trackService.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
    }

    @Transactional
    @Override
    public MarkLibrary save(MarkLibrary library) {
        return libraryDao.save(library);
    }

    @Override
    public MarkLibrary findById(int id) {
        return libraryDao.findOne(id);
    }

    @Override
    public MarkLibrary findByStudentAndGroup(int studentId, int groupNumber) {
        List<MarkLibrary> list = libraryDao.findByStudentIdAndGroupNumber(studentId, groupNumber);
        return list != null && list.size() > 0 ? list.get(0) : null;
    }

    @Override
    public List<MarkLibrary> findByStudentId(int studentId) {
        List<MarkLibrary> list = libraryDao.findByStudentId(studentId);
        // return list != null && list.size() > 0 ? list.get(0) : null;
        return list;
    }

    @Override
    public List<MarkLibrary> findByStatus(int examId, String subjectCode, int groupNumber, LibraryStatus status,
            int pageNumber, int pageSize) {
        MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
        query.setPageNumber(pageNumber);
        query.setPageSize(pageSize);
        return libraryDao.findByExamIdAndSubjectCodeAndGroupNumberAndStatus(examId, subjectCode, groupNumber, status,
                query);
    }

    @Override
    public List<MarkLibrary> findByStatusSet(int examId, String subjectCode, int groupNumber,
            Set<LibraryStatus> statusSet, int pageNumber, int pageSize) {
        MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
        query.setPageNumber(pageNumber);
        query.setPageSize(pageSize);
        return libraryDao.findByExamIdAndSubjectCodeAndGroupNumberAndStatusIn(examId, subjectCode, groupNumber,
                statusSet, query);
    }

    @Override
    @Transactional
    public void resetByGroup(MarkGroup group) {
    	markSpecialTagService.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
        libraryDao.resetByExamIdAndSubjectCodeAndNumber(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                LibraryStatus.WAITING);
        trackService.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
    }

    @Override
    @Transactional
    public void resetByMarker(Marker marker) {
        libraryDao.resetByMarkerId(marker.getId(), LibraryStatus.WAITING);
        trackService.deleteByMarkerId(marker.getId());
        markSpecialTagService.deleteByMarkerId(marker.getId());
    }

    @Override
    @Transactional
    public void resetById(int id) {
        libraryDao.resetById(id, LibraryStatus.WAITING);
        MarkLibrary library = findById(id);
        if (library != null) {
            List<ExamQuestion> examQuestions = questionService.findByExamAndSubjectAndObjectiveAndMainNumber(
                    library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
            for (ExamQuestion examQuestion : examQuestions) {
                trackService.deleteByStudentIdAndQuestionNumber(library.getStudentId(),
                        examQuestion.getQuestionNumber());
            }
            markSpecialTagService.deleteByLibraryId(id);
        }
    }

    @Override
    @Transactional
    public void backById(int id) {
        libraryDao.resetById(id, LibraryStatus.BACKED);
        MarkLibrary library = findById(id);
        if (library != null) {
            List<ExamQuestion> examQuestions = questionService.findByExamAndSubjectAndObjectiveAndMainNumber(
                    library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
            for (ExamQuestion examQuestion : examQuestions) {
                trackService.deleteByStudentIdAndQuestionNumber(library.getStudentId(),
                        examQuestion.getQuestionNumber());
            }
            markSpecialTagService.deleteByLibraryId(id);
        }
    }

    @Override
    public List<Integer> findTagCampusId(int examId) {
        return libraryDao.findTagCampusId(examId);
    }

    @Override
    public List<String> findTagSubjectCode(int examId) {
        return libraryDao.findTagSubjectCode(examId);
    }

    @Override
    public long countByExamAndSubjectAndGroupAndStatus(int examId, String subjectCode, int groupNumber,
            LibraryStatus status) {
        MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setGroupNumber(groupNumber);
        query.setStatus(status);
        return countByQuery(query);
    }

    @Override
    public long countByMarker(int markerId) {
        MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
        query.setMarkerId(markerId);
        query.setStatus(LibraryStatus.MARKED);
        return countByQuery(query);
    }

    @Override
    public long countByQuery(final MarkLibrarySearchQuery query) {
        return libraryDao.count(buildSpecification(query));
    }

    @Override
    public MarkLibrarySearchQuery findByQuery(final MarkLibrarySearchQuery query) {
        checkQuery(query);
        Page<MarkLibrary> result = libraryDao.findAll(buildSpecification(query), query);
        fillResult(result, query);
        return query;
    }

    private Specification<MarkLibrary> buildSpecification(final MarkLibrarySearchQuery query) {
        return new Specification<MarkLibrary>() {

            @Override
            public Predicate toPredicate(Root<MarkLibrary> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getExamId() > 0) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (query.getCampusId() > 0) {
                    predicates.add(cb.equal(root.get("campusId"), query.getCampusId()));
                }
                if (query.getGroupNumber() > 0) {
                    predicates.add(cb.equal(root.get("groupNumber"), query.getGroupNumber()));
                }
                if (StringUtils.isNotBlank(query.getExamNumber())) {
                    predicates.add(cb.equal(root.get("examNumber"), query.getExamNumber()));
                }
                if (query.getStudentId() > 0) {
                    predicates.add(cb.equal(root.get("studentId"), query.getStudentId()));
                }
                if (query.getMarkerId() > 0) {
                    predicates.add(cb.equal(root.get("markerId"), query.getMarkerId()));
                }
                if (query.getStatus() != null) {
                    predicates.add(cb.equal(root.get("status").as(LibraryStatus.class), query.getStatus()));
                }
                if (query.getTagNotNull() != null && query.getTagNotNull().booleanValue()) {
                    predicates.add(cb.isNotNull(root.get("tags")));
                }
                if (query.getTagId() > 0) {
                    predicates.add(cb.like(root.get("tags").as(String.class), "%" + query.getTagId() + "%"));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        };
    }

    @Override
    public List<MarkLibrary> findByStudentIdAndStatus(int studentId, LibraryStatus status) {
        return libraryDao.findByStudentIdAndStatus(studentId, status);
    }
}
