package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import cn.com.qmth.stmms.biz.utils.MarkRedisUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.campus.model.Campus;
import cn.com.qmth.stmms.biz.campus.service.CampusService;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.MarkSpecialTag;
import cn.com.qmth.stmms.biz.mark.model.MarkStepDTO;
import cn.com.qmth.stmms.biz.mark.model.MarkTrack;
import cn.com.qmth.stmms.biz.mark.model.SpecialTagDTO;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.model.TrackDTO;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.biz.utils.CurrentTaskUtil;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.utils.PictureUrlBuilder;

/**
 * 评卷任务服务实现
 * 
 * @author LS
 * 
 */
@Service("taskService")
public class TaskServiceImpl implements TaskService {

    private static Logger log = LoggerFactory.getLogger(TaskServiceImpl.class);

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private MarkGroupService groupService;
    
    @Autowired
    private MarkSpecialTagService markSpecialTagService;
    
    @Autowired
    private CampusService campusService;
    
    @Override
    public List<Task> findByQuery(MarkLibrarySearchQuery query) {
        List<Task> list = new LinkedList<Task>();
        query = libraryService.findByQuery(query);
        if (query.getCurrentCount() > 0) {
            for (MarkLibrary library : query.getResult()) {
                list.add(build(library));
            }
        }
        return list;
    }

    @Override
    public Task build(MarkLibrary library) {
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());
        Task task = new Task();
        task.setTagList(getMarkSpecialTagList(library.getId()));
        task.setExist(true);
        task.setStudentId(library.getExamNumber());
        task.setLibraryId(library.getId());
        task.setMarkStepList(buildMarkStep(library));
        task.setPictureUrls(PictureUrlBuilder.getSliceUrls(library.getExamId(), library.getCampusId(),
                library.getSubjectCode(), library.getExamNumber(), group.getPicNumbers()));
        task.setSheetUrls(PictureUrlBuilder.getSheetUrls(library.getExamId(), library.getCampusId(),
                library.getSubjectCode(), library.getExamNumber(), student.getSheetCount()));
        task.setAnswerUrl(PictureUrlBuilder.getAnswerUrl(library.getExamId(), library.getSubjectCode()));
        task.setPaperUrl(PictureUrlBuilder.getPaperUrl(library.getExamId(), library.getSubjectCode()));
        task.setObjectiveScore(student != null ? student.getObjectiveScore() : 0);
        task.setMarkTime(library.getMarkerTime());
        task.setTags(library.getTags());
        if (library.getMarkerId() != null) {
            task.setMarkId(library.getMarkerId());
        }
        if (library.getMarkerScore() != null) {
            task.setTotalScore(library.getMarkerScore());
        }
        if (library.getMarkerScoreList() != null) {
            task.setScoreList(library.getMarkerScoreList());
        }
        if (library.getStatus() == LibraryStatus.BACKED && library.getMarkerId() != null) {
            task.setBack(true);
        }
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkLibrary library) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndMainNumber(library.getExamId(),
                library.getSubjectCode(), false, library.getGroupNumber());
        int number = 0;
        for (ExamQuestion question : sList) {
            number++;

            MarkStepDTO step = new MarkStepDTO();
            step.setQuestionNumber(question.getQuestionNumber());
            step.setBlockId(question.getMainNumber());
            step.setNumber(number);
            step.setTitle(question.getMainTitle() + "-" + question.getSubNumber());
            step.setTotalScore(question.getTotalScore());
            step.setDefaultScore(0d);
            step.setHasLevel(false);
            step.setMax(question.getTotalScore());
            step.setMin(0d);
            step.setInterval(question.getIntervalScore());
            step.setScoreList(question.getScoreListArray());
            // 增加阅卷轨迹列表获取
            addTrack(step, question, trackService.findByStudentId(library.getStudentId()));
            list.add(step);
        }
        return list;
    }

    private void addTrack(MarkStepDTO step, ExamQuestion question, List<MarkTrack> trackList) {
        String questionNumber = question.getQuestionNumber();
        for (MarkTrack track : trackList) {
            if (track.getQuestionNumber().equals(questionNumber)) {
                step.addTrack(new TrackDTO(track));
            }
        }
    }

    @Override
    @Transactional
    public boolean submit(Task task) {
        if (task == null) {
            return false;
        }
        try {
            submitLibrary(task);
            return true;
        } catch (Exception e) {
            log.error("task submit faile", e);
            return false;
        }
    }

    private void submitLibrary(Task task) {
        MarkLibrary library = updateLibrary(task);
        if (library == null) {
            return;
        }
        // 同步操作
        if (task.isProblem()) {
            // ProblemLibrary library = new ProblemLibrary();
            // library.setExamId(formallyLibrary.getExamId());
            // library.setCampusCode(formallyLibrary.getCampusCode());
            // library.setSubjectCode(formallyLibrary.getSubjectCode());
            // library.setLibraryId(formallyLibrary.getId());
            // library.setType(LibraryType.FORMALLY);
            // library.setBlockId(formallyLibrary.getBlockId());
            // library.setSecretNo(formallyLibrary.getSecretNo());
            // library.setReason(task.getReason());
            // library.setSubmitTime(new Date());
            // library.setSubmitter(task.getMarkId());
            // library.setStage(formallyLibrary.getMarkerCount());
            // library.setStatus(LibraryStatus.WAITING);
            // problemLibraryService.save(library);
        } else {
            // FormallyHistory history = new FormallyHistory();
            // history.setLibraryId(task.getLibraryId());
            // history.setScoreList(task.getScoreList());
            // history.setScorer(task.getMarkId());
            // history.setScoreTime(new Date());
            // history.setSpent(task.getSpent());
            // history.setStage(task.getMarkerCount());
            // history.setTotalScore(task.getTotalScore());
            // if (task.isSelf()) {
            // history.setType(HistoryType.SELF);
            // } else if (task.isBack()) {
            // history.setType(HistoryType.BACK);
            // } else {
            // history.setType(HistoryType.COMMON);
            // BlockTaskCountUtil.addBlockMarkedTask(stringRedisTemplate,
            // task.getBlockId(), task.getLibraryId());
            // }
            // formallyHistoryService.save(history);

            onTaskSubmit(library, task);
        }
    }

    private void onTaskSubmit(MarkLibrary library, Task task) {
        if (library.getStatus() == LibraryStatus.MARKED) {
            // ExamStudent student =
            // studentService.findById(library.getStudentId());
            // if (student != null) {
            // student.setSubjectiveScore(library.getMarkerScore());
            // student.setScoreList(library.getScoreList(), false);
            // studentService.save(student);
            // }
            List<MarkGroup> groups = groupService.findByExamAndSubject(library.getExamId(), library.getSubjectCode());
            List<MarkLibrary> libraries = libraryService.findByStudentIdAndStatus(library.getStudentId(),
                    LibraryStatus.MARKED);
            if (libraries.size() == groups.size()) {
                List<ScoreItem> scoreList = new ArrayList<ScoreItem>();
                Double markerScore = 0.0;
                for (MarkLibrary librarie : libraries) {
                    scoreList.addAll(librarie.getScoreList());
                    markerScore = markerScore + librarie.getMarkerScore();
                }
                studentService.updateSubjectiveScore(library.getStudentId(), markerScore,
                        ExamStudent.buildScoreList(scoreList));
            }

            groupService.updateLibraryCount(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

            Map<String, List<TrackDTO>> trackMap = task.getTrackMap();
            for (String questionNumber : trackMap.keySet()) {
                trackService.deleteByStudentIdAndQuestionNumber(library.getStudentId(), questionNumber);
                List<TrackDTO> list = trackMap.get(questionNumber);
                for (TrackDTO dto : list) {
                    trackService.save(dto.transform(library));
                }
            }

            SpecialTagDTO [] tagList = task.getTagList();
            markSpecialTagService.deleteByLibraryId(library.getId());
            if(tagList != null && tagList.length>0){
                for(SpecialTagDTO s : tagList){
                        MarkSpecialTag markSpecialTag = new MarkSpecialTag();
                        markSpecialTag.setLibraryId(library.getId());
                        markSpecialTag.setTagName(s.getTagName());
                        markSpecialTag.setPositionX(s.getPositionX());
                        markSpecialTag.setPositionY(s.getPositionY());
                        markSpecialTagService.save(markSpecialTag);
                }
            }
        }
    }

    /**
     * 保存评卷任务
     * 
     * @param task
     * @return
     */
    private MarkLibrary updateLibrary(Task task) {
        MarkLibrary library = libraryService.findById(task.getLibraryId());
        if (library != null) {
            library.setMarkerId(task.getMarkId());
            library.setMarkerTime(new Date());
            library.setTags(StringUtils.trimToNull(task.getTags()));
            library.setStatus(LibraryStatus.MARKED);
            if (!task.isProblem()) {
                library.setMarkerScore(task.getTotalScore());
                library.setMarkerScoreList(task.getScoreList());
            }
            return libraryService.save(library);
        } else {
            return null;
        }
    }

    @Override
    public boolean setCurrent(Marker marker, int libraryId) {
        return CurrentTaskUtil.add(marker, libraryId);
    }

    @Override
    public void removeCurrent(Marker marker, int libraryId) {
        CurrentTaskUtil.remove(marker, libraryId);
    }

    @Override
    public int countCurrent(int examId, String subjectCode, int number) {
        return CurrentTaskUtil.count(examId, subjectCode, number);
    }

    @Override
    public int countCurrent(Marker marker) {
        return CurrentTaskUtil.count(marker);
    }

    @Override
    public void clearCurrent(int examId, String subjectCode, int number) {
        CurrentTaskUtil.clear(examId, subjectCode, number);
    }

    @Override
    public void clearCurrent(Marker marker) {
        CurrentTaskUtil.clear(marker);
    }
    
    public SpecialTagDTO [] getMarkSpecialTagList(Integer libraryId){
        SpecialTagDTO [] specialTags = null;
        List<MarkSpecialTag> list =  markSpecialTagService.findByLibraryId(libraryId);
        if(!list.isEmpty()){
            specialTags = new SpecialTagDTO[list.size()] ;
            for(int i =0;i<list.size();i++){
                SpecialTagDTO specialTagDTO = new SpecialTagDTO();
                specialTagDTO.setTagName(list.get(i).getTagName());
                specialTagDTO.setPositionX(list.get(i).getPositionX());
                specialTagDTO.setPositionY(list.get(i).getPositionY());
                specialTags[i] = specialTagDTO;
            }
        }
        return  specialTags;
    }

	@Override
	public Task build(Integer studentId) {
        ExamStudent student = studentService.findById(studentId);
        Task task = new Task();
        task.setExist(true);
        task.setStudentId(student.getExamNumber());
        task.setLibraryId(studentId);
        task.setMarkStepList(buildMarkStep(student));
        Campus campus = campusService.findBySchoolAndName(student.getSchoolId(), student.getCampusName());
        task.setPictureUrls(PictureUrlBuilder.getSliceUrls(student.getExamId(), campus.getId(),
        		student.getSubjectCode(), student.getExamNumber(), 1, student.getSliceCount()-1));
        task.setSheetUrls(PictureUrlBuilder.getSheetUrls(student.getExamId(), campus.getId(),
        		student.getSubjectCode(), student.getExamNumber(), student.getSheetCount()));
        task.setAnswerUrl(PictureUrlBuilder.getAnswerUrl(student.getExamId(), student.getSubjectCode()));
        task.setPaperUrl(PictureUrlBuilder.getPaperUrl(student.getExamId(), student.getSubjectCode()));
        task.setObjectiveScore(student != null ? student.getObjectiveScore() : 0);
        task.setBack(true);
        return task;
    }
	private List<MarkStepDTO> buildMarkStep(ExamStudent student) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
        		student.getSubjectCode(), false);
        int number = 0;
        for (ExamQuestion question : sList) {
            number++;

            MarkStepDTO step = new MarkStepDTO();
            step.setQuestionNumber(question.getQuestionNumber());
            step.setBlockId(question.getMainNumber());
            step.setNumber(number);
            step.setTitle(question.getMainTitle() + "-" + question.getSubNumber());
            step.setTotalScore(question.getTotalScore());
            step.setDefaultScore(0d);
            step.setHasLevel(false);
            step.setMax(question.getTotalScore());
            step.setMin(0d);
            step.setInterval(question.getIntervalScore());
            step.setScoreList(question.getScoreListArray());
            // 增加阅卷轨迹列表获取
            addTrack(step, question, trackService.findByStudentId(student.getId()));
            list.add(step);
        }
        return list;
    }
	
	@Transactional
	@Override
	public boolean submitByStudent(Task task) {
		if (task == null) {
            return false;
        }
        try {
        	List<MarkLibrary> libraries = libraryService.findByStudentId(task.getLibraryId());
        	int start = 0 ;
        	int end = 0;
        	for (int i = 0; i < libraries.size(); i++) {
        		MarkLibrary library = libraries.get(i);
        		Task stepTask =new Task();
        		stepTask.setProblem(task.isProblem());
        		stepTask.setBack(task.isBack());
        		stepTask.setLibraryId(library.getId());
        		stepTask.setMarkId(task.getMarkId());
        		stepTask.setTags(task.getTags());
        		List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndMainNumber(library.getExamId(),
                        library.getSubjectCode(), false, library.getGroupNumber());
        		if(start==end ){
        			end = end + sList.size()*2-1;
        		}else{
        			start = end+1;
        			end = start + sList.size()*2-1;
        		}
        		String scoreList = task.getScoreList().substring(start, end);
        		stepTask.setScoreList(scoreList);
        		String[] scores = scoreList.split(",");
        		double totalScore = 0;
        		for (String score : scores) {
        			totalScore = totalScore + Double.parseDouble(score);
				}
         		stepTask.setTotalScore(totalScore);
        		submitLibrary(stepTask);
			}
            return true;
        } catch (Exception e) {
            log.error("task submit faile", e);
            return false;
        }
	}

    /*
     * 客户端非正常情况下退出系统时，对内存中的没有及时清除的任务进行处理
     *
     */
    @Override
    public void clearTaskMap(long cleanMapinterval) throws Exception{
            CurrentTaskUtil.QuartzClearMap(cleanMapinterval);
    }

}
