package cn.com.qmth.stmms.admin.exam;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.dto.MarkGroupDTO;
import cn.com.qmth.stmms.admin.dto.MarkerInfoDTO;
import cn.com.qmth.stmms.admin.vo.SubjectLibraryVO;
import cn.com.qmth.stmms.biz.campus.model.Campus;
import cn.com.qmth.stmms.biz.campus.service.CampusService;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.ExamSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller("markInfoController")
@RequestMapping("/admin/exam/mark")
public class MarkController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(MarkController.class);

    @Autowired
    private ExamService examService;

    @Autowired
    private CampusService campusService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private MarkGroupService groupService;

    /**
     * 评卷进度
     * 
     * @param request
     * @param model
     * @param query
     * @return
     */
    @RequestMapping(value = { "", "/subject" })
    public String subjectList(HttpServletRequest request, Model model, ExamSubjectSearchQuery query) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        // query.setUploadCountGt(0);
        subjectFilter(query, wu);
        query = subjectService.findByQuery(query);

        List<SubjectLibraryVO> list = new LinkedList<SubjectLibraryVO>();
        for (ExamSubject subject : query.getResult()) {
            SubjectLibraryVO vo = new SubjectLibraryVO();
            vo.setSubject(subject);
            vo.setStudentCount(subject.getUploadCount());
            vo.setGroupCount(groupService.countByExamAndSubject(examId, subject.getCode()));
            list.add(vo);
        }

        double total = libraryService.countByExamAndSubjectAndGroupAndStatus(examId, null, 0, null);
        double finish = libraryService.countByExamAndSubjectAndGroupAndStatus(examId, null, 0, LibraryStatus.MARKED);
        double percent = total > 0 ? (finish * 100 / total) : 0;
        model.addAttribute("percent", new DecimalFormat("###.#").format(percent));
        model.addAttribute("resultList", list);
        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        model.addAttribute("levelList", subjectService.listLevel(examId));
        model.addAttribute("categoryList", subjectService.listCategory(examId));
        return "modules/exam/markInfo";
    }

    @RequestMapping(value = "/export-progress", method = RequestMethod.POST)
    public String exportProgress(HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        List<MarkGroupDTO> result = new LinkedList<MarkGroupDTO>();
        List<MarkGroup> list = wu.isSubjectHeader()
                ? groupService.findByExamAndSubject(examId, wu.getUser().getSubjectCode())
                : groupService.findByExam(examId);
        for (MarkGroup group : list) {
            group.setMarkerCount(
                    markerService.countByExamAndSubjectAndGroup(examId, group.getSubjectCode(), group.getNumber()));
            MarkGroupDTO dto = new MarkGroupDTO(subjectService.find(examId, group.getSubjectCode()), group);
            result.add(dto);
        }

        try {
            String fileName = "整体评卷进度.xlsx";
            new ExportExcel("整体评卷进度", MarkGroupDTO.class).setDataList(result).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出整体评卷进度失败！" + e.getMessage());
            return "redirect:/admin/exam/mark";
        }
    }

    @RequestMapping(value = "/export-marker", method = RequestMethod.POST)
    public String exportMarker(HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        List<MarkerInfoDTO> result = new LinkedList<MarkerInfoDTO>();
        List<Marker> list = wu.isSubjectHeader() ? markerService.getMarkCount(examId, wu.getUser().getSubjectCode())
                : markerService.getMarkCount(examId);
        for (Marker marker : list) {
            MarkerInfoDTO dto = new MarkerInfoDTO(marker,
                    subjectService.find(marker.getExamId(), marker.getSubjectCode()),
                    groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber()));
            result.add(dto);
        }

        try {
            String fileName = "评卷员工作量.xlsx";
            new ExportExcel("评卷员工作量", MarkerInfoDTO.class).setDataList(result).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出评卷员工作量失败！" + e.getMessage());
            return "redirect:/admin/exam/mark";
        }
    }

    /**
     * 定时生成评卷任务
     */
    @Scheduled(fixedDelay = 2 * 60 * 1000, initialDelay = 60 * 1000)
    public void createLibrary() {
        log.debug("start auto-create library");
        try {
            Map<String, Campus> campusMap = new HashMap<String, Campus>();

            ExamSearchQuery query = new ExamSearchQuery();
            query.addStatus(ExamStatus.START);
            query.setSort(new Sort(Direction.DESC, "id"));
            query.setPageNumber(1);
            query.setPageSize(10);
            query = examService.findByQuery(query);
            while (query.getCurrentCount() > 0) {
                for (Exam exam : query.getResult()) {
                    createLibraryByExam(exam, campusMap);
                }
                query.setPageNumber(query.getPageNumber() + 1);
                query = examService.findByQuery(query);
            }
        } catch (Exception e) {
        } finally {
            log.debug("finish auto-create library");
        }
    }

    private void createLibraryByExam(Exam exam, Map<String, Campus> campusMap) {
        List<ExamSubject> subjects = subjectService.list(exam.getId(), 0);
        for (ExamSubject subject : subjects) {
            // 清除缺考考生和违纪考生
            List<Integer> idList = studentService.findAbsentOrBreachLibraryStudent(subject.getExamId(), subject.getCode());
            if (idList != null) {
                for (Integer studentId : idList) {
                    libraryService.deleteByStudent(studentId);
                }
                groupService.updateLibraryCount(subject.getExamId(), subject.getCode());
            }
            // 处理正常考生
            List<MarkGroup> groups = groupService.findByExamAndSubject(exam.getId(), subject.getCode());
            for (MarkGroup group : groups) {
                Date lastBuildTime = group.getBuildTime();
                ExamStudent student = studentService.findUnLibraryStudent(exam.getId(), subject.getCode(),
                        group.getNumber(), lastBuildTime);
                while (student != null) {
                    MarkGroup current = groupService.findOne(group.getExamId(), group.getSubjectCode(),
                            group.getNumber());
                    if (current == null || (current.getBuildTime() == null && lastBuildTime != null)) {
                        // 大题已被删除，或者已被重置，直接退出循环
                        break;
                    }
                    Campus campus = campusMap.get(student.getSchoolId() + "_" + student.getCampusName());
                    if (campus == null) {
                        campus = campusService.findBySchoolAndName(student.getSchoolId(), student.getCampusName());
                        if (campus == null) {
                            log.error("campus unexist for student id=" + student.getId());
                            continue;
                        } else {
                            campusMap.put(student.getSchoolId() + "_" + student.getCampusName(), campus);
                        }
                    }
                    MarkLibrary library = libraryService.findByStudentAndGroup(student.getId(), group.getNumber());
                    if (library == null) {
                        library = new MarkLibrary();
                        library.setExamId(student.getExamId());
                        library.setSubjectCode(student.getSubjectCode());
                        library.setGroupNumber(group.getNumber());
                        library.setCampusId(campus.getId());
                        library.setStudentId(student.getId());
                        library.setExamNumber(student.getExamNumber());
                        library.setStatus(LibraryStatus.WAITING);
                        libraryService.save(library);
                    }
                    lastBuildTime = student.getUploadTime();
                    groupService.updateBuildTime(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                            lastBuildTime);
                    groupService.updateLibraryCount(exam.getId(), subject.getCode(), group.getNumber());
                    // 取下一个考生
                    student = studentService.findUnLibraryStudent(exam.getId(), subject.getCode(), group.getNumber(),
                            lastBuildTime);
                }
            }
        }
    }

}
