package cn.com.qmth.stmms.api.controller;

import java.text.DecimalFormat;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.google.common.base.Strings;

import cn.com.qmth.stmms.api.utils.AESUtil;
import cn.com.qmth.stmms.biz.api.auth.annotation.AuthValidate;
import cn.com.qmth.stmms.biz.campus.model.Campus;
import cn.com.qmth.stmms.biz.campus.service.CampusService;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamPackage;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamPackageService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

@Controller("examStudentApiController")
@RequestMapping("/api")
public class ExamStudentController {

    protected static Logger logger = LoggerFactory.getLogger(ExamStudentController.class);

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private ExamService examService;

    @Autowired
    private CampusService campusService;

    @Autowired
    private ExamPackageService packageService;

    @RequestMapping(value = "/package/{examId}", method = RequestMethod.GET)
    @ResponseBody
    public JSONArray getPackageCode(HttpServletRequest request, @PathVariable Integer examId) {
        JSONArray array = new JSONArray();
        Exam exam = examService.findById(examId);
        if (exam == null) {
            return array;
        }

        List<String> list = examStudentService.findDistinctPackageCode(examId);
        if (list != null) {
            for (String code : list) {
                array.add(code);
            }
        }
        return array;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/package/{examId}", method = RequestMethod.POST)
    @ResponseBody
    public int updatePackage(HttpServletRequest request, @PathVariable Integer examId,
            @RequestBody ExamPackage examPackage) {
        Exam exam = examService.findById(examId);
        if (exam != null) {
            ExamPackage obj = packageService.find(examId, examPackage.getCode());
            if (obj != null && examPackage.getPicCount() != null) {
                obj.setPicCount(examPackage.getPicCount());
                obj = packageService.save(obj);
                return obj.getPicCount();
            }
        }
        return -1;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/exam/students/{examId}", method = RequestMethod.GET)
    @ResponseBody
    public JSONArray getExamStudents(HttpServletRequest request, @PathVariable Integer examId) {
        JSONArray array = new JSONArray();
        List<ExamStudent> esList = new LinkedList<ExamStudent>();
        Exam exam = examService.findById(examId);
        if (exam != null) {
            esList = examStudentService.findByExamId(examId);
        }
        for (ExamStudent student : esList) {
            JSONObject obj = new JSONObject();
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentId", String.valueOf(student.getId()));
            obj.accumulate("barcode", student.getExamNumber());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
            array.add(obj);
        }
        return array;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/students/{examId}", method = RequestMethod.GET)
    @ResponseBody
    public JSONArray getStudent(HttpServletRequest request, @PathVariable Integer examId,
            @RequestParam(required = false) Boolean upload, @RequestParam(required = false) Boolean absent,
            @RequestParam(required = false) Integer pageNumber, @RequestParam(required = false) Integer pageSize) {
        JSONArray array = new JSONArray();
        Exam exam = examService.findById(examId);
        if (exam == null) {
            return array;
        }

        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setUpload(upload);
        query.setAbsent(absent);
        if (pageNumber != null && pageSize != null) {
            query.setPageNumber(pageNumber);
            query.setPageSize(pageSize);
        } else {
            query.setPageNumber(1);
            query.setPageSize(Integer.MAX_VALUE);
        }
        examStudentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            JSONObject obj = new JSONObject();
            obj.accumulate("id", student.getId());
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentCode", student.getStudentCode());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
            array.add(obj);
        }
        return array;
    }

    @AuthValidate({ "adminUser", "scanner" })
    @RequestMapping(value = "/student/check", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject checkStudent(HttpServletRequest request, @RequestBody ExamStudent examStudent) {
        JSONObject obj = new JSONObject();
        Exam exam = examService.findById(examStudent.getExamId());
        ExamStudent student = examStudentService.findByExamIdAndExamNumber(examStudent.getExamId(),
                examStudent.getExamNumber());
        if (student != null) {
            obj.accumulate("examId", examStudent.getExamId());
            obj.accumulate("campusName", student.getCampusName());
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("name", student.getName());
            obj.accumulate("studentId", String.valueOf(student.getId()));
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());

            Campus campus = campusService.findBySchoolAndName(exam.getSchoolId(), student.getCampusName());
            obj.accumulate("campusCode", campus != null ? campus.getId().toString() : "");
        } else {
            obj.accumulate("examId", examStudent.getExamId());
            obj.accumulate("campusCode", "");
            obj.accumulate("campusName", "");
            obj.accumulate("examNumber", examStudent.getExamNumber());
            obj.accumulate("name", "");
            obj.accumulate("studentId", "");
            obj.accumulate("subjectCode", "");
            obj.accumulate("subjectName", "");
        }
        return obj;
    }

    /**
     * 
     * @param s
     *            需要转换的字符串
     * @param convert
     *            是否正向转换
     * @return
     */
    @SuppressWarnings("unused")
    private String convert(String s, String[] diploma, String[] bachelorDegree, boolean convert) {
        if (diploma == null || bachelorDegree == null) {
            return s;
        }

        char[] ss = s.toCharArray();
        String[] str = new String[ss.length];
        for (int i = 0; i < str.length; i++) {
            str[i] = String.valueOf(ss[i]);
        }

        if (convert) {
            if (str[Integer.parseInt(diploma[2]) - 1].equals(diploma[1])) {
                str[Integer.parseInt(diploma[2]) - 1] = diploma[0];
            }
            if (str[Integer.parseInt(diploma[3]) - 1].equals(diploma[1])) {
                str[Integer.parseInt(diploma[3]) - 1] = diploma[0];
            }
            if (str[Integer.parseInt(bachelorDegree[2]) - 1].equals(bachelorDegree[1])) {
                str[Integer.parseInt(bachelorDegree[2]) - 1] = bachelorDegree[0];
            }
            if (str[Integer.parseInt(bachelorDegree[3]) - 1].equals(bachelorDegree[1])) {
                str[Integer.parseInt(bachelorDegree[3]) - 1] = bachelorDegree[0];
            }
        } else {
            if (str[Integer.parseInt(diploma[2]) - 1].equals(diploma[0])) {
                str[Integer.parseInt(diploma[2]) - 1] = diploma[1];
            }
            if (str[Integer.parseInt(diploma[3]) - 1].equals(diploma[0])) {
                str[Integer.parseInt(diploma[3]) - 1] = diploma[1];
            }
            if (str[Integer.parseInt(bachelorDegree[2]) - 1].equals(bachelorDegree[0])) {
                str[Integer.parseInt(bachelorDegree[2]) - 1] = bachelorDegree[1];
            }
            if (str[Integer.parseInt(bachelorDegree[3]) - 1].equals(bachelorDegree[0])) {
                str[Integer.parseInt(bachelorDegree[3]) - 1] = bachelorDegree[1];
            }
        }
        StringBuffer sb = new StringBuffer();
        for (String i : str) {
            sb.append(i);
        }
        return sb.toString();
    }

    @RequestMapping(value = "/score/school/{schoolId}", method = RequestMethod.POST)
    @ResponseBody
    public String getScore(@PathVariable Integer schoolId, @RequestParam String studentCode,
            @RequestParam String subjectCode, @RequestParam(required = false, defaultValue = "true") boolean encrypt,
            @RequestParam(required = false) String examSeqCode) {
        JSONObject obj = new JSONObject();
        ExamStudent student = null;
        try {
            if (Strings.isNullOrEmpty(examSeqCode)) {
                student = examStudentService.findBySchoolIdAndSubjectCodeAndStudentCode(schoolId, subjectCode,
                        studentCode);
            } else {
                student = examStudentService.findBySchoolIdAndSubjectCodeAndStudentCodeAndRemark(schoolId, subjectCode,
                        studentCode, examSeqCode);
            }
            if (student != null) {
                DecimalFormat df = new DecimalFormat("###.#");
                obj.accumulate("exist", true);
                obj.accumulate("objectiveScore", df.format(student.getObjectiveScore()));
                obj.accumulate("subjectiveScore", df.format(student.getSubjectiveScore()));
                obj.accumulate("totalScore", df.format(student.getTotalScore()));
            } else {
                obj.accumulate("exist", false);
                obj.accumulate("objectiveScore", 0);
                obj.accumulate("subjectiveScore", 0);
                obj.accumulate("totalScore", 0);
            }
            if (!Strings.isNullOrEmpty(examSeqCode)) {
                obj.accumulate("examSeqCode", examSeqCode);
            }
            obj.accumulate("studentCode", studentCode);
            obj.accumulate("subjectCode", subjectCode);
        } catch (Exception e) {
            e.printStackTrace();
            obj.accumulate("exception", "500");
        }
        String result = obj.toString();
        return encrypt ? AESUtil.encrypt(result) : result;
    }
}
